package require XOTcl
namespace import ::xotcl::*

package require rete
namespace import ::rete::*

::rete::WorkingMemory workingMem

namespace eval ::sample {
	#
	# Simple Model...
	#
	Class Address -slots {
	  Attribute street 
	  Attribute zip
	}
	
	Class Company -slots {
	  Attribute name
	  Attribute location
	  Attribute address -type ::sample::Address
	}
	
	Class Person -slots {
	  Attribute name 
	  Attribute age
	  Attribute salary
	  Attribute employed -type ::sample::Company
	}
	
	
	#
	# ... with a few facts ....
	#
	Address a1 -street Augasse1 -zip 1090
	Address a2 -street street1 -zip 00123
	Address a3 -street Sollingergasse24 -zip 1190
	Address a4 -street TemporaryRoad12 -zip 0007
	
	Company c1 -name KM -location Vienna -address ::sample::a1
	Company c2 -name AS -location US -address ::sample::a2
	Company c3 -name WU -location Wien -address ::sample::a1
	Company c4 -name KGemeinschaft -location Vienna -address ::sample::a3
	Company c5 -name WU -location Vienna -address ::sample::a1
	
	Person p1 -name Bernd -age 31 -salary 100 -employed ::sample::c1
	Person p2 -name Gustaf -age 77 -salary 120 -employed ::sample::c3
	Person p3 -name Franz -age 22 -salary 80 -employed ::sample::c3
	Person p4 -name Jeff -age 55 -salary 120 -employed ::sample::c2
	Person p5 -name Philip -age 11 -salary 90 -employed ::sample::c4
	Person p6 -name Mark -age 31 -salary 100 -employed ::sample::c5 
	
	#	
	# ok, we are done. Define a materialized View
	# Workplace...
	#
	set query {[$p employed] eq $c && [$c location] eq "Vienna" && [$c address] eq $a && [$a zip] >= 1090 }
	puts "\nrun Query1: '$query'"
	
		 
	View Workplace as {
	  forall {::sample::Person p ::sample::Company c ::sample::Address a} {
	    my set name [$p name]
		my set company [$c name]  
	    my set location [$c location]
		my set address [$a street]
		my set salary [$p set salary 1]
	  } where {
	     [$p employed] eq $c && [$c location] eq "Vienna" && [$c address] eq $a && [$a zip] >= 1090 
	  }
	}
	
	#
	# Show the results
	#
	puts "print results of Query 1"
	foreach w [Workplace info instances] {
	  puts  stderr "$w company '[$w set company]' address '[$w set address]' location '[$w set location]' employee '[$w set name]' newSalary '[$w set salary]' "
	}
	
	#
	# okay try another query
	#
	
	set query {[$c location] eq "Vienna" && [$c address] eq $a && [$a zip] >= 1090 && [string length [$c name]] > 2 }
	puts "\nrun Query2: '$query'"
	
	View CompanyInVienna as {
	  forall {Company c Address a} {
		my set company [string toupper [$c name]]  
	    my set location [$c location]
		my set address [$a street]
	  } where {
	     ([$c location] eq "Vienna" || [$c location] eq "Wien") && [$c address] eq $a && [$a zip] >= 1090 && [string length [$c name]] > 2
	  }
	}
	
		 
	#
	# Show the results
	#
	puts "print results of Query 2"
	foreach w [CompanyInVienna info instances] {
	   puts  stderr "$w company '[$w set company]' address '[$w set address]' location '[$w set location]' "
	}
	
	set query3 {[$a zip] >= 1090}
	puts "\nrun Query3: '$query3'"
	View AdressOver1090 as {
	  forall {Address a} {
		my set address [$a street]
		my set zip [$a zip]
	  } where {
	     [$a zip] >= 1090
	  }
	}
	puts "print results of Query 3"
	foreach w [AdressOver1090 info instances] {
	   puts  stderr " $w address '[$w set address]' zip '[$w set zip]' "
	}
	
	puts "\t adding some classes"
	Address a4 -street somehwere1 -zip 2020; #overwrite
	Address a5 -street somehwere1 -zip 2020
	Address a6 -street somehwere1 -zip 2020
	Address a7 -street somehwere1 -zip 2020
	
	workingMem refreshClass Address
	
	View AdressOver1090Repeat as {
	  forall {Address a} {
		my set address [$a street]
		my set zip [$a zip]
	  } where {
	     [$a zip] >= 1090
	  }
	}
		
	puts "print results of Query 3 repeat"
	foreach w [AdressOver1090Repeat info instances] {
	  puts  stderr " $w address '[$w set address]' zip '[$w set zip]' "
	}
	
	puts "\nold Query 3 stays the same"
	foreach w [AdressOver1090 info instances] {
	  puts  stderr " $w address '[$w set address]' zip '[$w set zip]' "
	}

}